/*
 * PIMPPA - Create all assign patterns
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <ctype.h>
#include <unistd.h>

#include "pimppa.h"

int ClearPrevious=0;					// Clear previous assign patterns?
int DeleteZero=0;						// Delete those with target 0
int DeleteNeg=0;						// Delete those with target -1
				
void usage(char *name)
{
	fprintf(stderr, "Usage: %s <options>\n\n"
				"-c     Clear all previous assigns. Does not affect\n"
				"       patterns with destinations 0 or -1.\n"
				"-0     Delete all assign patterns with target area 0.\n"
				"-n     Delete all assign patterns with target area -1.\n"
				,name);
}	

int create_assigns(void)
{
	MYSQL src_db;
	MYSQL dst_db;
	MYSQL_RES *sql_res;
	MYSQL_ROW sql_row;
	unsigned long total_files=0;
	char *value;
	int minassnamelen;

/********* Connect databases ****************/

	if(!p_connect(&src_db))
		return(0);

	if(!p_connect(&dst_db))
	{
		mysql_close(&src_db);
		return(0);
	}

	value=p_getmisc(&src_db, P_KEY_MINASSNAMELEN);
	if(value)
		minassnamelen=atoi(value);
	else
		minassnamelen=atoi(P_MIN_ASS_NAMELENGTH);

/******** Delete previous patterns? ********/

	if(ClearPrevious)
	{
		fprintf(stderr, "Clearing previous assign patterns...\n");
		p_query(&dst_db, "DELETE FROM p_assign WHERE a_dest>0");
	}
	
	if(DeleteZero)
	{
		fprintf(stderr, "Deleting assign patterns with target 0...\n");
		p_query(&dst_db, "DELETE FROM p_assign WHERE a_dest=0");
	}

	if(DeleteNeg)
	{
		fprintf(stderr, "Deleting assign patterns with target -1...\n");
		p_query(&dst_db, "DELETE FROM p_assign WHERE a_dest=-1");
	}

/******** check and assign *******/

	fprintf(stderr, "Processing filenames...\n");

	p_query(&src_db, "SELECT file_name, file_area
					FROM p_files, p_areas
					WHERE NOT (area_flags & %ld) AND NOT (area_flags & %ld)
					AND area_id=file_area
					AND NOT (file_backup=0 AND (file_flags & %ld))",
					AREA_INCOMING, AREA_NOASSIGN, FILE_OFFLINE);
	if(mysql_error(&src_db)[0])
		return(-1);
	sql_res=mysql_use_result(&src_db);

	while((sql_row=mysql_fetch_row(sql_res)))
	{
		if(p_assign(&dst_db, sql_row[0], minassnamelen, atoi(sql_row[1])))
		{
			total_files++;
		
			if(total_files%500==0)
				fprintf(stderr, "%ld done...\n", total_files);
		}
	}

	mysql_free_result(sql_res);

	fprintf(stderr, "Counting totals...\n");

	p_query(&src_db, "SELECT count(*) FROM p_assign
					WHERE a_dest>=0");
	sql_res=mysql_store_result(&src_db);
	if(sql_res)
	{
		sql_row=mysql_fetch_row(sql_res);
		
		fprintf(stderr, "Parsed %ld files mapping to %s patterns.\n",
			total_files, sql_row[0]);
		
		mysql_free_result(sql_res);
	}

	mysql_close(&src_db);
	mysql_close(&dst_db);

	return(total_files);
}
	

int main(int argc, char *argv[])
{
	int go=1;

	while(go)
	{
		switch(getopt(argc, argv, "0chnV"))
		{
			case '0':
				DeleteZero=1;
				break;
			case 'n':
				DeleteNeg=1;
				break;
			case 'c':
				ClearPrevious=1;
				break;
			case 'h':
				usage(argv[0]);
				return(0);
				break;
			case 'V':
				printf("%s %s %s\n", PACKAGE, argv[0], VERSION);
				return(0);
				break;
			default:
				go=0;
				break;
		}
	}	

	create_assigns();
	
	return(0);
}

