/*
 * News downloading shell util for PIMPPA
 *
 *
 * Uses: news.c (and indirectly suck, uudeview)
 *
 *
 */

#include <errno.h>
#include <time.h>
#include <ctype.h>
#include <dirent.h>
#include <getopt.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/vfs.h>

#include "pimppa.h"
#include "ftp.h"
#include "news.h"
	
int WaitA, WaitS;

MYSQL src_db,dst_db;

void usage(char *name)
{
	fprintf(stderr, "Usage: %s [options]\n\n"
			"-a <n>       Wait After n messages, def=%d\n"
			"-f <urlfile> Read (ftp-url, area) pairs from file, leech recursively\n" 
			"-g <pat>     Newsgroup name SQL pattern to match, def=all defined\n"
			"-H <host>    Specify NNTP host.\n"
			"-l           Lenient article selection while leeching news\n"
			"-n           Nazi behaviour, delete all leeched files which\n"
			"             do not have an assign pattern already defined.\n"
			"-U <user>    NNTP server username, if needed by the server\n"
			"-P <passwd>  NNTP server password, if needed by the server\n"
			"-q           Quiet operation (don't print newsleech BPS etc)\n"
			"-r           Restart newsgroup(s)\n"
			"-v           Verbose output\n"
			"-w <secs>    Wait secs after [-a n] messages, def=%d\n",
		name,WaitA, WaitS);
}


int main(int argc, char *argv[])
{
	char server[PATH_MAX];
	char groups[PATH_MAX];
	char urlfile[PATH_MAX];
	int go=1;
	int options=0;
	int retval=0;
	char username[256];
	char passwd[256];
	char *value;

	server[0]=groups[0]=username[0]=passwd[0]=urlfile[0]=0;
	
	if(!p_connect(&src_db))
	{
		fprintf(stderr, "Error: %s\n", mysql_error(&src_db));
		return(-1);
	}
	value=p_getmisc(&src_db, P_KEY_NEWSWAITAFTER);
	if(value)
		WaitA=atoi(value);
	else
		WaitA=atoi(P_NEWS_WAIT_AFTER);
	value=p_getmisc(&src_db, P_KEY_NEWSWAITSECS);
	if(value)
		WaitS=atoi(value);
	else
		WaitS=atoi(P_NEWS_WAIT_SECS);

	while(go)
	{
		switch(getopt(argc, argv, "f:g:a:hH:lnsU:P:qrvVw:"))
		{
			case 'a':
				WaitA=atoi(optarg);
				break;
			case 'f':
				strcpy(urlfile, optarg);
				break;
			case 'g':
				strcpy(groups, optarg);
				break;
			case 'H':
				strcpy(server, optarg);
				break;
			case 'l':
				options|=OPT_LENIENT;
				break;
			case 'n':
				options|=OPT_NAZI;
				break;
			case 'U':
				strcpy(username, optarg);
				break;
			case 'P':
				strcpy(passwd, optarg);
				break;
			case 'q':
				options|=OPT_QUIET;
				break;
			case 'r':
				options|=OPT_RESTART;
				break;
			case 'v':
				options|=OPT_VERBOSE;
				break;
			case 'V':
				printf("%s %s %s\n", PACKAGE, argv[0], VERSION);
				mysql_close(&src_db);
				return(0);
				break;
			case 'w':
				WaitS=atoi(optarg);
				break;
			case -1:
				go=0;
				break;
			case 'h':
			default:
				usage(argv[0]);
				mysql_close(&src_db);
				return(0);
				break;
		}
	}

	if(!p_connect(&dst_db))
	{
		mysql_close(&src_db);
		fprintf(stderr, "Error: %s\n", mysql_error(&dst_db));
		return(-1);
	}

	if(urlfile[0])
	{
		retval=p_ftpleech(urlfile, options);
	
		mysql_close(&src_db);
		mysql_close(&dst_db);
		
		return(retval);
	}
	
	// Else newsleech
	
	if(!server[0])   // No arg, check if a news server is set in "p_misc"
	{
		char *tmp;
		
		tmp=p_getmisc(&src_db, P_KEY_NNTPSERVER);
		if(tmp)
			strcpy(server, tmp);
		else
			p_setmisc(&dst_db, P_KEY_NNTPSERVER, P_DEFAULT_NNTPSERVER); 
		
		if(!server[0])		// Not in db? Use one in pimppa.h then
			strcpy(server, P_DEFAULT_NNTPSERVER);
	}
	
	retval=p_dlnews(server, groups, username, passwd, options, 
		WaitA, WaitS);

	mysql_close(&src_db);
	mysql_close(&dst_db);

	return(retval);
}

